"""Relang.

Self hosted, simplest possible web interface for text translations using Meta's
NLLB200 model."""

__version__ = "0.1"


def cli():
    "Target for [project.scripts]."
    from . import __main__



from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
from flask import Flask, render_template, request, Response

from .model import Model
from .lang import LANGUAGE_CODES

parser = ArgumentParser(prog="relang", formatter_class=ArgumentDefaultsHelpFormatter)
parser.add_argument(
    "sp_path", help="path to the tokenization model in sentencepiece format"
)
parser.add_argument("tr_path", help="path to the NLLB model in CTranslate2 format")
parser.add_argument(
    "--batchsize",
    metavar="B",
    type=int,
    default=2,
    help="number of batches to process in parallel",
)
parser.add_argument(
    "--threads",
    metavar="T",
    type=int,
    default=12,
    help="number of threads to use per batch",
)
parser.add_argument(
    "--beamsize",
    metavar="B",
    type=int,
    default=4,
    help="number of hypotheses to consider at all times",
)
parser.add_argument(
    "--host", metavar="H", type=str, default="0.0.0.0", help="the interface to bind to"
)
parser.add_argument(
    "--port", metavar="P", type=int, default=5000, help="the port to bind to"
)
parser.add_argument(
    "--gpu",
    dest="device",
    default="cpu",
    action="store_const",
    const="cuda",
    help="use the GPU, requires CUDA",
)
args = parser.parse_args()

model = Model(
    args.sp_path,
    args.tr_path,
    device=args.device,
    intra_threads=args.threads,
)

app = Flask("relang")


@app.route("/")
def route_index():
    return render_template("index.html", languages=LANGUAGE_CODES)


@app.route("/translate", methods=["POST"])
def route_translate():
    post = request.get_json()
    event_stream = model.translate_text(
        src_text=post["src_text"],
        src_lang=post["src_lang"],
        tgt_lang=post["tgt_lang"],
        max_batch_size=args.batchsize,
        beam_size=args.beamsize,
    )
    return Response(event_stream, mimetype="text/event-stream")


app.run(host=args.host, port=args.port)



LANGUAGE_CODES = {
    "ace_Arab": "Acehnese (Arabic script)",
    "ace_Latn": "Acehnese (Latin script)",
    "acm_Arab": "Arabic, Mesopotamian",
    "acq_Arab": "Arabic, Ta’izzi-Adeni",
    "aeb_Arab": "Arabic, Tunisian",
    "afr_Latn": "Afrikaans",
    "ajp_Arab": "Arabic, South Levantine",
    "aka_Latn": "Akan",
    "als_Latn": "Tosk Albanian",
    "amh_Ethi": "Amharic",
    "apc_Arab": "Arabic, North Levantine",
    "arb_Arab": "Arabic, Modern Standard",
    "ars_Arab": "Arabic, Najdi",
    "ary_Arab": "Arabic, Moroccan",
    "arz_Arab": "Arabic, Egyptian",
    "asm_Beng": "Assamese",
    "ast_Latn": "Asturian",
    "awa_Deva": "Awadhi",
    "ayr_Latn": "Central Aymara",
    "azb_Arab": "South Azerbaijani",
    "azj_Latn": "North Azerbaijani",
    "bak_Cyrl": "Bashkir",
    "bam_Latn": "Bambara",
    "ban_Latn": "Balinese",
    "bel_Cyrl": "Belarusian",
    "bem_Latn": "Bemba",
    "ben_Beng": "Bengali",
    "bho_Deva": "Bhojpuri",
    "bjn_Arab": "Banjar (Arabic script)",
    "bjn_Latn": "Banjar (Latin script)",
    "bod_Tibt": "Standard Tibetan",
    "bos_Latn": "Bosnian",
    "bug_Latn": "Buginese",
    "bul_Cyrl": "Bulgarian",
    "cat_Latn": "Catalan",
    "ceb_Latn": "Cebuano",
    "ces_Latn": "Czech",
    "cjk_Latn": "Chokwe",
    "ckb_Arab": "Central Kurdish",
    "crh_Latn": "Crimean Tatar",
    "cym_Latn": "Welsh",
    "dan_Latn": "Danish",
    "deu_Latn": "German",
    "dik_Latn": "Southwestern Dinka",
    "dyu_Latn": "Dyula",
    "dzo_Tibt": "Dzongkha",
    "ell_Grek": "Greek",
    "eng_Latn": "English",
    "epo_Latn": "Esperanto",
    "est_Latn": "Estonian",
    "eus_Latn": "Basque",
    "ewe_Latn": "Ewe",
    "fao_Latn": "Faroese",
    "fij_Latn": "Fijian",
    "fin_Latn": "Finnish",
    "fon_Latn": "Fon",
    "fra_Latn": "French",
    "fur_Latn": "Friulian",
    "fuv_Latn": "Nigerian Fulfulde",
    "gaz_Latn": "West Central Oromo",
    "gla_Latn": "Scottish Gaelic",
    "gle_Latn": "Irish",
    "glg_Latn": "Galician",
    "grn_Latn": "Guarani",
    "guj_Gujr": "Gujarati",
    "hat_Latn": "Haitian Creole",
    "hau_Latn": "Hausa",
    "heb_Hebr": "Hebrew",
    "hin_Deva": "Hindi",
    "hne_Deva": "Chhattisgarhi",
    "hrv_Latn": "Croatian",
    "hun_Latn": "Hungarian",
    "hye_Armn": "Armenian",
    "ibo_Latn": "Igbo",
    "ilo_Latn": "Ilocano",
    "ind_Latn": "Indonesian",
    "isl_Latn": "Icelandic",
    "ita_Latn": "Italian",
    "jav_Latn": "Javanese",
    "jpn_Jpan": "Japanese",
    "kab_Latn": "Kabyle",
    "kac_Latn": "Jingpho",
    "kam_Latn": "Kamba",
    "kan_Knda": "Kannada",
    "kas_Arab": "Kashmiri (Arabic script)",
    "kas_Deva": "Kashmiri (Devanagari script)",
    "kat_Geor": "Georgian",
    "kaz_Cyrl": "Kazakh",
    "kbp_Latn": "Kabiyè",
    "kea_Latn": "Kabuverdianu",
    "khk_Cyrl": "Halh Mongolian",
    "khm_Khmr": "Khmer",
    "kik_Latn": "Kikuyu",
    "kin_Latn": "Kinyarwanda",
    "kir_Cyrl": "Kyrgyz",
    "kmb_Latn": "Kimbundu",
    "kmr_Latn": "Northern Kurdish",
    "knc_Arab": "Central Kanuri (Arabic script)",
    "knc_Latn": "Central Kanuri (Latin script)",
    "kon_Latn": "Kikongo",
    "kor_Hang": "Korean",
    "lao_Laoo": "Lao",
    "lij_Latn": "Ligurian",
    "lim_Latn": "Limburgish",
    "lin_Latn": "Lingala",
    "lit_Latn": "Lithuanian",
    "lmo_Latn": "Lombard",
    "ltg_Latn": "Latgalian",
    "ltz_Latn": "Luxembourgish",
    "lua_Latn": "Luba-Kasai",
    "lug_Latn": "Ganda",
    "luo_Latn": "Luo",
    "lus_Latn": "Mizo",
    "lvs_Latn": "Standard Latvian",
    "mag_Deva": "Magahi",
    "mai_Deva": "Maithili",
    "mal_Mlym": "Malayalam",
    "mar_Deva": "Marathi",
    "min_Latn": "Minangkabau (Latin script)",
    "mkd_Cyrl": "Macedonian",
    "mlt_Latn": "Maltese",
    "mni_Beng": "Meitei (Bengali script)",
    "mos_Latn": "Mossi",
    "mri_Latn": "Maori",
    "mya_Mymr": "Burmese",
    "nld_Latn": "Dutch",
    "nno_Latn": "Norwegian Nynorsk",
    "nob_Latn": "Norwegian Bokmål",
    "npi_Deva": "Nepali",
    "nso_Latn": "Northern Sotho",
    "nus_Latn": "Nuer",
    "nya_Latn": "Nyanja",
    "oci_Latn": "Occitan",
    "ory_Orya": "Odia",
    "pag_Latn": "Pangasinan",
    "pan_Guru": "Eastern Panjabi",
    "pap_Latn": "Papiamento",
    "pbt_Arab": "Southern Pashto",
    "pes_Arab": "Western Persian",
    "plt_Latn": "Plateau Malagasy",
    "pol_Latn": "Polish",
    "por_Latn": "Portuguese",
    "prs_Arab": "Dari",
    "quy_Latn": "Ayacucho Quechua",
    "ron_Latn": "Romanian",
    "run_Latn": "Rundi",
    "rus_Cyrl": "Russian",
    "sag_Latn": "Sango",
    "san_Deva": "Sanskrit",
    "sat_Olck": "Santali",
    "scn_Latn": "Sicilian",
    "shn_Mymr": "Shan",
    "sin_Sinh": "Sinhala",
    "slk_Latn": "Slovak",
    "slv_Latn": "Slovenian",
    "smo_Latn": "Samoan",
    "sna_Latn": "Shona",
    "snd_Arab": "Sindhi",
    "som_Latn": "Somali",
    "sot_Latn": "Southern Sotho",
    "spa_Latn": "Spanish",
    "srd_Latn": "Sardinian",
    "srp_Cyrl": "Serbian",
    "ssw_Latn": "Swati",
    "sun_Latn": "Sundanese",
    "swe_Latn": "Swedish",
    "swh_Latn": "Swahili",
    "szl_Latn": "Silesian",
    "tam_Taml": "Tamil",
    "taq_Latn": "Tamasheq (Latin script)",
    "taq_Tfng": "Tamasheq (Tifinagh script)",
    "tat_Cyrl": "Tatar",
    "tel_Telu": "Telugu",
    "tgk_Cyrl": "Tajik",
    "tgl_Latn": "Tagalog",
    "tha_Thai": "Thai",
    "tir_Ethi": "Tigrinya",
    "tpi_Latn": "Tok Pisin",
    "tsn_Latn": "Tswana",
    "tso_Latn": "Tsonga",
    "tuk_Latn": "Turkmen",
    "tum_Latn": "Tumbuka",
    "tur_Latn": "Turkish",
    "twi_Latn": "Twi",
    "tzm_Tfng": "Central Atlas Tamazight",
    "uig_Arab": "Uyghur",
    "ukr_Cyrl": "Ukrainian",
    "umb_Latn": "Umbundu",
    "urd_Arab": "Urdu",
    "uzn_Latn": "Northern Uzbek",
    "vec_Latn": "Venetian",
    "vie_Latn": "Vietnamese",
    "war_Latn": "Waray",
    "wol_Latn": "Wolof",
    "xho_Latn": "Xhosa",
    "ydd_Hebr": "Eastern Yiddish",
    "yor_Latn": "Yoruba",
    "yue_Hant": "Yue Chinese",
    "zho_Hans": "Chinese (Simplified)",
    "zho_Hant": "Chinese (Traditional)",
    "zsm_Latn": "Standard Malay",
    "zul_Latn": "Zulu",
}



from ctranslate2 import Translator
from sentencepiece import SentencePieceProcessor
import re


class Model:
    def __init__(self, sp_path, tr_path, **tr_args):
        self.sp = SentencePieceProcessor()
        self.sp.load(sp_path)
        self.tr = Translator(tr_path, **tr_args)
        self.charmap = str.maketrans({"“": '"', "”": '"', "–": "-"})
        self.intersent = re.compile(
            r"""(
              (?<=[.?!؟])   # positive lookbehind: punctuation
              \s+           # match: whitespace
              (?![a-zа-џ])  # negative lookahead: lower case
            | \s*\n\s*\n\s* # or match: double newline
            )""",
            re.VERBOSE,
        )

    def translate_text(self, src_text, **kwargs):
        parts = self.intersent.split(src_text.translate(self.charmap))
        sentences = parts[::2]
        spaces = *parts[1::2], ""
        for sentence, space in zip(
            self.translate_sentences(sentences, **kwargs), spaces
        ):
            yield sentence + space

    def translate_sentences(self, src_sents, *, src_lang, tgt_lang, **kwargs):
        for translated_sentence in self.tr.translate_iterable(
            [(src_lang, *s, "</s>") for s in self.sp.encode_as_pieces(src_sents)],
            batch_type="tokens",
            target_prefix=[[tgt_lang]] * len(src_sents),
            **kwargs,
        ):
            h = translated_sentence.hypotheses[0]
            yield self.sp.decode(h[1 if h and h[0] == tgt_lang else 0 :])
